// ZenLib::ZenTypes - To be independant of platform & compiler
// Copyright (C) 2002-2003 Jrme Martinez, Zen@MediaArea.net
//
// This software is provided 'as-is', without any express or implied
// warranty.  In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//    claim that you wrote the original software. If you use this software
//    in a product, an acknowledgment in the product documentation would be
//    appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//    misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// ZenTypes
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Version 0.3.0
// -------------
// Independancies of types
//               (u)int  (u)int  (u)int  (u)int  Float   Float
//                8bit   16bit   32bit   64bit   6digit  15digi
// Win32        :   X      X       X       X       X       X   
// Linux        :   X      X       X
// Platform detection
// __WIN32__, __UNIX__
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// 0.3.1 (Zen@MediaArea.net, 2003-06-22)
// Add Float
//
// 0.3.0 (Zen@MediaArea.net, 2003-06-22)
// Platform definition
//
// 0.2.0 (Zen@MediaArea.net)
// Win32 64 bits
// Linux 8-32 bits
//
// 0.1.0 (Zen@MediaArea.net)
// Win32 8-32 bits
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//---------------------------------------------------------------------------
#ifndef ZenTypesH
#define ZenTypesH
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#include <limits.h>
//---------------------------------------------------------------------------

namespace ZenLib
{

//***************************************************************************
// Platforms
//***************************************************************************

//---------------------------------------------------------------------------
//Win32 (Win95, Win98, WinMe, WinNT, WinXP)
#if (defined(__NT__) || defined(_WIN32) || defined(WIN32)) && !defined(__WIN32__)
    #define __WIN32__
#endif

//Unix (Linux, HP, Sun, BeOS...)
#if (defined(__unix) || defined(__unix__) || \
      defined(____SVR4____) || defined(__LINUX__) || defined(__sgi) || \
      defined(__hpux) || defined(sun) || defined(__SUN__) || defined(_AIX) || \
      defined(__EMX__) || defined(__VMS) || defined(__BEOS__)) && !defined(__UNIX__)
#define __UNIX__
#endif

//Test of targets
#if defined(__WIN32__) && defined(__UNIX__)
#error Two platforms???
#endif

#if !defined(__WIN32__) && !defined(__UNIX__)
#warning No known platforms, assume default
#endif

//***************************************************************************
// Types
//***************************************************************************

//---------------------------------------------------------------------------
//8-bit int
#if UCHAR_MAX==0xff
    #undef MAXTYPE_INT
    #define MAXTYPE_INT 8
    typedef unsigned char   uint8;
    typedef signed char      int8;
#else
    #error This machine has no 8-bit type
#endif

//---------------------------------------------------------------------------
//16-bit int
#if UINT_MAX == 0xffff
    #undef MAXTYPE_INT
    #define MAXTYPE_INT 16
    typedef unsigned int    uint16;
    typedef int              int16;
#elif USHRT_MAX == 0xffff
    #undef MAXTYPE_INT
    #define MAXTYPE_INT 16
    typedef unsigned short  uint16;
    typedef short            int16;
#else
    #error This machine has no 16-bit type
#endif

//---------------------------------------------------------------------------
//32-bit int
#if UINT_MAX == 0xfffffffful
    #undef MAXTYPE_INT
    #define MAXTYPE_INT 32
    typedef unsigned int    uint32;
    typedef int              int32;
#elif ULONG_MAX == 0xfffffffful
    #undef MAXTYPE_INT
    #define MAXTYPE_INT 32
    typedef unsigned long   uint32;
    typedef long             int32;
#elif USHRT_MAX == 0xfffffffful
    #undef MAXTYPE_INT
    #define MAXTYPE_INT 32
    typedef unsigned short  uint32;
    typedef short            int32;
#else
    #error This machine has no 32-bit type
#endif

//---------------------------------------------------------------------------
//64-bit int
#if defined(__MINGW32__) || defined(__CYGWIN32__)
    #undef MAXTYPE_INT
    #define MAXTYPE_INT 64
    typedef long long          int64; //not tested!!!
    typedef unsigned long long uint64; //not tested!!!
#else
    #ifdef __WIN32__
        #undef MAXTYPE_INT
        #define MAXTYPE_INT 64
        typedef __int64          int64;
        typedef unsigned __int64 uint64;
    #endif
#endif

//---------------------------------------------------------------------------
//5-digit float
//#if FLT_DIG >= 5 || _MSC_VER //Microsoft doesn't know FLT_DIG???
	#undef MAXTYPE_FLOAT
    #define MAXTYPE_FLOAT 32
    typedef float		    float5;
//#else
//    #error This machine has no 5-digit float type
//#endif

//---------------------------------------------------------------------------
//15-digit float
//#if DBL_DIG >= 15 || _MSC_VER //Microsoft doesn't know DBL_DIG???
	#undef MAXTYPE_FLOAT
    #define MAXTYPE_FLOAT 32
    typedef double		    float15;
//#else
//    #error This machine has no 5-digit float type
//#endif

} //namespace
#endif
