// ZenLib::FileB - Binary file manipulation
// Copyright (C) 2002-2003 Jrme Martinez, Zen@MediaArea.net
//
// This software is provided 'as-is', without any express or implied
// warranty.  In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//    claim that you wrote the original software. If you use this software
//    in a product, an acknowledgment in the product documentation would be
//    appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//    misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Version 0.0.1
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// 0.1.0.001 (Zen@MediaArea.net, 2003-06-14)
// Warning : will change in future
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//---------------------------------------------------------------------------
#include <ZenLib/FileB.h>
#include <string.h>
#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <windows.h>
//---------------------------------------------------------------------------

namespace ZenLib
{

//***************************************************************************
// Gestion de la classe
//***************************************************************************

//---------------------------------------------------------------------------
//Constructeur
FileB::FileB ()
{
    F=-1;
    End=true;
    Nom="";
}

//---------------------------------------------------------------------------
//Destructeur
FileB::~FileB ()
{
    Fermer();
}

//***************************************************************************
// Ouverture/fermeture
//***************************************************************************

//---------------------------------------------------------------------------
//Ouverture du fichier

int FileB::Ouvrir (const Ztring &Fichier, bool Nouveau, bool Ajout, bool Ecriture)
{
    //Si fichier ouvert on ferme
    Fermer();

    //Recuperation nom du fichier a ouvrir
    if (Fichier!="")
    	Nom=Fichier;

    //Ouverture fichier
    if (Nouveau)
        F=open(Nom.Char(), O_RDWR|O_CREAT|O_TRUNC|O_BINARY, S_IREAD|S_IWRITE); //Nouveau fichier
    else if (Ajout)
        F=open(Nom.Char(), O_RDWR|O_CREAT|O_APPEND|O_BINARY, S_IREAD|S_IWRITE); //Veut ecrire en ajout
    else if (Ecriture)
        F=open(Nom.Char(), O_RDWR|O_CREAT|O_BINARY, S_IREAD|S_IWRITE); //Veux ecrire
    else
        F=open(Nom.Char(), O_RDONLY|O_BINARY, S_IREAD|S_IWRITE);
    if (F<0)
        return -1; //---Si pas de fichier correct

    //Proprietes
    End=false;

    return 1;
}

//---------------------------------------------------------------------------
//Fermeture du fichier
void FileB::Fermer ()
{
    //Fermeture
    if (F>=0)
    	close(F);

    //Proprits
    F=-1;
    End=true;
}

//***************************************************************************
// Texte
//***************************************************************************

//---------------------------------------------------------------------------
//Operation de lecture
int FileB::Lire(char* Data, int Taille_Max)
{
    return Lire((unsigned char*)Data, Taille_Max);
}

int FileB::Lire(unsigned char* Data, int Taille_Max)
{
    //Test integrite
    if (F<0)
        return -1;

    //Lecture
    int I1=read(F, Data, Taille_Max);

    //Proprietes
    if (I1<Taille_Max || I1==Longueur())
        End=true;

    return I1;
}

int FileB::Lire(char* Data, int Taille_Max, int Offset, int Whence)
{
    return Lire((unsigned char*)Data, Taille_Max, Offset, Whence);
}

int FileB::Lire(unsigned char* Data, int Taille_Max, int Offset, int Whence)
{
    //Test integrite
    if (F<0)
        return -1;

   //Repositionnement?
   if (lseek(F, Offset, Whence)<0)
       return -2;

   return Lire(Data, Taille_Max);
}

//---------------------------------------------------------------------------
//opration d'criture
int FileB::Ecrire(const char* Data, int Taille_Max)
{
    return Ecrire((unsigned char*)Data, Taille_Max);
}

int FileB::Ecrire (const unsigned char* Data, int Taille)
{
   //Test integrite
   if (F<0)
       return -1;

   //Ecriture
   write(F, Data, Taille);

   return 1;
}

int FileB::Ecrire(const char* Data, int Taille_Max, int Offset, int Whence)
{
    return Ecrire((unsigned char*)Data, Taille_Max, Offset, Whence);
}

int FileB::Ecrire (const unsigned char* Data, int Taille, int Offset, int Whence)
{
   //Test integrite
   if (F<0)
       return -1;

   //Repositionnement?
   if (lseek(F, Offset, Whence)<0)
       return -2;

   return Ecrire(Data, Taille);
}

//***************************************************************************
// Divers
//***************************************************************************

//---------------------------------------------------------------------------
//End
bool FileB::Eof()
{
    return End;
}

int FileB::Longueur()
{
    if (F>=0)
        return filelength(F);
    else
        return -1;
}

int FileB::Supprimer(Ztring Fichier)
{
    if (DeleteFile(Fichier.Char()))
    	return 1;
    else
    	return -1;
}

int FileB::Position_Get()
{
    if (F<0)
	return -1;
    return tell(F);
}

int FileB::Position_Set(int Offset, int Whence)
{
    if (F<0)
	return -1;
    return lseek(F, Offset, Whence);
}

Ztring FileB::Nom_Get()
{
	return Nom;
}

} //Namespace
