// ZenLib::ZenUtils - Very small utilities
// Copyright (C) 2002-2003 Jrme Martinez, Zen@MediaArea.net
//
// This software is provided 'as-is', without any express or implied
// warranty.  In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//    claim that you wrote the original software. If you use this software
//    in a product, an acknowledgment in the product documentation would be
//    appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//    misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Version 0.1.0
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// 0.1.0.001 (Zen@MediaArea.net, 2003-03-26)
// BigEndianToInt (8-64 bits)
// LittleEndianToInt (8-64 bits)
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//---------------------------------------------------------------------------
#include <ZenLib/Utils.h>
//---------------------------------------------------------------------------

namespace ZenLib
{

//***************************************************************************
// Transformations
//***************************************************************************

//---------------------------------------------------------------------------
// Little Endian - 8 bits
 int8   LittleEndian2int8   (const uint8* Liste)
{
    int8 Retour=Liste[0];
    return Retour;
}

uint8   LittleEndian2uint8  (const uint8* Liste)
{
    uint8 Retour=Liste[0];
    return Retour;
}

//---------------------------------------------------------------------------
// Little Endian - 16 bits
 int16  LittleEndian2int16  (const uint8* Liste)
{
    int16 Retour=Liste[1];
    Retour=(Retour<<8)|Liste[0];
    return Retour;
}

uint16  LittleEndian2uint16 (const uint8* Liste)
{
    uint16 Retour=Liste[1];
    Retour=(Retour<<8)|Liste[0];
    return Retour;
}

//---------------------------------------------------------------------------
// Little Endian - 32 bits
 int32  LittleEndian2int32  (const uint8* Liste)
{
    int32 Retour=Liste[3];
    Retour=(Retour<<8)|Liste[2];
    Retour=(Retour<<8)|Liste[1];
    Retour=(Retour<<8)|Liste[0];
    return Retour;
}

uint32  LittleEndian2uint32 (const uint8* Liste)
{
    uint32 Retour=Liste[3];
    Retour=(Retour<<8)|Liste[2];
    Retour=(Retour<<8)|Liste[1];
    Retour=(Retour<<8)|Liste[0];
    return Retour;
}

//---------------------------------------------------------------------------
// Little Endian - 64 bits
 int64  LittleEndian2int64  (const uint8* Liste)
{
    int64 Retour=Liste[7];
    Retour=(Retour<<8)|Liste[6];
    Retour=(Retour<<8)|Liste[5];
    Retour=(Retour<<8)|Liste[4];
    Retour=(Retour<<8)|Liste[3];
    Retour=(Retour<<8)|Liste[2];
    Retour=(Retour<<8)|Liste[1];
    Retour=(Retour<<8)|Liste[0];
    return Retour;
}

uint64  LittleEndian2uint64 (const uint8* Liste)
{
    uint64 Retour=Liste[7];
    Retour=(Retour<<8)|Liste[6];
    Retour=(Retour<<8)|Liste[5];
    Retour=(Retour<<8)|Liste[4];
    Retour=(Retour<<8)|Liste[3];
    Retour=(Retour<<8)|Liste[2];
    Retour=(Retour<<8)|Liste[1];
    Retour=(Retour<<8)|Liste[0];
    return Retour;
}

//---------------------------------------------------------------------------
// Big Endian - 8 bits
 int8   BigEndian2int8      (const uint8* Liste)
{
    int8 Retour=Liste[0];
    return Retour;
}

uint8   BigEndian2uint8      (const uint8* Liste)
{
    uint8 Retour=Liste[0];
    return Retour;
}

//---------------------------------------------------------------------------
// Big Endian - 16 bits
 int16  BigEndian2int16      (const uint8* Liste)
{
    int16 Retour=Liste[0];
    Retour=(Retour<<8)|Liste[1];
    return Retour;
}

uint16  BigEndian2uint16     (const uint8* Liste)
{
    uint16 Retour=Liste[0];
    Retour=(Retour<<8)|Liste[1];
    return Retour;
}

//---------------------------------------------------------------------------
// Big Endian - 32 bits
 int32  BigEndian2int32      (const uint8* Liste)
{
    int32 Retour=Liste[0];
    Retour=(Retour<<8)|Liste[1];
    Retour=(Retour<<8)|Liste[2];
    Retour=(Retour<<8)|Liste[3];
    return Retour;
}

uint32  BigEndian2uint32     (const uint8* Liste)
{
    uint32 Retour=Liste[0];
    Retour=(Retour<<8)|Liste[1];
    Retour=(Retour<<8)|Liste[2];
    Retour=(Retour<<8)|Liste[3];
    return Retour;
}

//---------------------------------------------------------------------------
// Big Endian - 64 bits
 int64  BigEndian2int64      (const uint8* Liste)
{
    int64 Retour=Liste[0];
    Retour=(Retour<<8)|Liste[1];
    Retour=(Retour<<8)|Liste[2];
    Retour=(Retour<<8)|Liste[3];
    Retour=(Retour<<8)|Liste[4];
    Retour=(Retour<<8)|Liste[5];
    Retour=(Retour<<8)|Liste[6];
    Retour=(Retour<<8)|Liste[7];
    return Retour;
}

uint64  BigEndian2uint64     (const uint8* Liste)
{
    uint64 Retour=Liste[0];
    Retour=(Retour<<8)|Liste[1];
    Retour=(Retour<<8)|Liste[2];
    Retour=(Retour<<8)|Liste[3];
    Retour=(Retour<<8)|Liste[4];
    Retour=(Retour<<8)|Liste[5];
    Retour=(Retour<<8)|Liste[6];
    Retour=(Retour<<8)|Liste[7];
    return Retour;
}

//---------------------------------------------------------------------------
// int32 - int64
int64 int32_int64   (                int32 High,  uint32 Low)
{
    return (((( int64)High)<<32) | Low);
}

int64 uint32_uint64 (               uint32 High,  uint32 Low)
{
    return ((((uint64)High)<<32) | Low);
}

void  int64_int32   ( int64 BigInt,  int32 &High, uint32 &Low)
{
    High = ( int32) ((BigInt & 0xFFFFFFFF00000000)>>32);
    Low  = ( int32) ( BigInt & 0x00000000FFFFFFFF);
}

void  uint64_uint32 (uint64 BigInt, uint32 &High, uint32 &Low)
{
    High = (uint32) ((BigInt & 0xFFFFFFFF00000000)>>32);
    Low  = ( int32) ( BigInt & 0x00000000FFFFFFFF);
}

//---------------------------------------------------------------------------
// Floats and ints
int32  Float2int32  (float F, int Rounded)
{
    return (int)F;
}

} //namespace ZenLib


