// ZenLib::Ztring - std::string is better
// Copyright (C) 2002-2003 Jrme Martinez, Zen@MediaArea.net
//
// This software is provided 'as-is', without any express or implied
// warranty.  In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//    claim that you wrote the original software. If you use this software
//    in a product, an acknowledgment in the product documentation would be
//    appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//    misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Version 0.0.1
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//++++++++++++++++++++++++.Replace+++++++++++++++++++++++++++++++++++++++++++++++++++

//---------------------------------------------------------------------------
#include <ZenLib/Ztring.h>
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#ifdef __BORLANDC__ //Warnings in Borland
#pragma warn -8059  //
#endif              //
#include <algorithm>
#include <functional>
#ifdef __BORLANDC__ //Warnings in Borland
#pragma warn .8059  //
#endif              //
//---------------------------------------------------------------------------

namespace ZenLib
{
//static char Chaine_Bidon='\0';

//***************************************************************************
// Gestion de la classe
//***************************************************************************

//---------------------------------------------------------------------------
//Constructeurs
Ztring::Ztring (const int32 I, size_type radix)
{
    //Conversion
    char* C1=new char[33]; //taille maxi d'un int en base 10 en characteres
    itoa (I, C1, 10);

    //Remplissage
    append(C1);
    delete[] C1;
}

Ztring::Ztring (const uint32 I, size_type radix)
{
    //Conversion
    char* C1=new char[33]; //taille maxi d'un int en base 10 en cahracteres
    ultoa (I, C1, 10);

    //Remplissage
    append(C1);
    delete[] C1;
}

#if (MAXTYPE_INT >= 64)
Ztring::Ztring (const int64 I, size_type radix)
{
    //Conversion
    char* C1=new char[66]; //taille maxi d'un int en base 10 en characteres
    _i64toa (I, C1, radix);

    //Remplissage
    append(C1);
    delete[] C1;
}

Ztring::Ztring (const uint64 I, size_type radix)
{
    //Conversion
    char* C1=new char[66]; //taille maxi d'un int en base 10 en cahracteres
    _ui64toa (I, C1, radix);

    //Remplissage
    append(C1);
    delete[] C1;
}
#endif

Ztring::Ztring (const float F)
{
    //Conversion
    char C1[66]; //taille maxi d'un int en base 10 en cahracteres
    sprintf (C1, "%.f", F);

    //Remplissage
    append(C1);
}

Ztring::Ztring (const double F)
{
    //Conversion
    char C1[66]; //taille maxi d'un int en base 10 en cahracteres
    sprintf (C1, "%.f", F);

    //Remplissage
    append(C1);
}

//***************************************************************************
// Oprateurs
//***************************************************************************

//---------------------------------------------------------------------------
//Operateur +
Ztring Ztring::operator + (const Ztring &Source) const
{
    Ztring C2(this);
    C2.append(Source);

    return C2;
}

Ztring operator + (const char* Avant, const Ztring &Source)
{
    Ztring C2(Avant);
    C2.append(Source);

    return C2;
}

//---------------------------------------------------------------------------
//Operateur +=
Ztring &Ztring::operator += (const Ztring &Source)
{
    append(Source.c_str());
    return *this;
}

Ztring &Ztring::operator += (const char *Source)
{
    Ztring S(Source);
    append(S);
    return *this;
}

//***************************************************************************
// Conversions
//***************************************************************************

//---------------------------------------------------------------------------
//Operateur ToChar
char* Ztring::ToChar () const
{
    //Test integrite
    if (empty())
    {
        char* C1=new char[1];
        C1[0]='\0';
        return C1;
    }

    //Remplissage
    char* C1=new char[length()+1];
    strcpy (C1, c_str());
    return C1;
}


//---------------------------------------------------------------------------
//Operateur ToInt
int Ztring::Convert_int32 (ztring_t Options) const
{
    //Test integrite
    if (empty())
        return 0;

    if (Options==Ztring_Rounded)
    {
        int32 I=atoi(c_str());
        float15 F=Convert_float15();
        F-=I;
        if (F<0.5)
            return I;
        else
            return I+1;
    }
    return (atoi(c_str()));
}

//---------------------------------------------------------------------------
//Operateur ToInt
int64 Ztring::Convert_int64 (ztring_t Options) const
{
    //Test integrite
    if (empty())
        return 0;

    if (Options==Ztring_Rounded)
    {
        int64 I=_atoi64(c_str());
        float15 F=Convert_float15();
        F-=I;
        if (F<0.5)
            return I;
        else
            return I+1;
    }
    return (atoi(c_str()));
}

//---------------------------------------------------------------------------
//Operateur ToInt
uint64 Ztring::Convert_uint64 (ztring_t Options) const
{
    //Test integrite
    if (empty())
        return 0;

    if (Options==Ztring_Rounded)
    {
        uint64 I=_atoi64(c_str());
        float15 F=Convert_float15();
        F-=I;
        if (F<0.5)                   
            return I;
        else
            return I+1;
    }
    return (atoi(c_str()));
}


//---------------------------------------------------------------------------
//Operateur ToFloat
float5 Ztring::Convert_float5(ztring_t Options) const
{
    //Test integrite
    if (empty())
        return 0;

    return ((float5)atof(c_str()));
}

//---------------------------------------------------------------------------
//Operateur ToFloat
float15 Ztring::Convert_float15(ztring_t Options) const
{
    //Test integrite
    if (empty())
        return 0;

    return (atof(c_str()));
}

/*
//---------------------------------------------------------------------------
//Operateur ToInt (arrondi)
int Ztring::FloatToInt_Rounded (float F)
{
    int I=F;
    F-=I;
    if (F<0.5)
        return I;
    else
        return I+1;
}

*/
//***************************************************************************
// Edition
//***************************************************************************

//---------------------------------------------------------------------------
// Retourne une partie de la chaine
Ztring Ztring::SubString (const std::string &Begin, const std::string &End, int Pos, ztring_t Options) const
{
    //Recherche Dbut
    int I_Debut=find(Begin, Pos);
    if (I_Debut==-1)
        return "";
    I_Debut+=Begin.size();

    //gestion fin NULL
    if (End=="")
        return substr(I_Debut);

    //Recherche Fin
    int I_Fin=find(End, I_Debut);
    if (I_Fin==-1)
    {
        if (Options & Ztring_AddLastItem)
            return substr(I_Debut);
        else
            return "";
    }

    return substr(I_Debut, I_Fin-I_Debut);
}

/*
//---------------------------------------------------------------------------
//Recherche
int Ztring::Rechercher(const Ztring &Trouver, int Pos_Debut, int Pos_Fin, int Options) const
{
    //Test integrite
    if (!&Trouver || !Trouver.Longueur())
        return Pos_Debut;
    if (!Chaine || !Chaine[0])
        return -1;
    if (Pos_Debut<0 && Pos_Fin<0)
        return -1;
    if (Longueur()<Trouver.Longueur())
        return -1;
        

    //Mise de Debut et Fin comme il faut (sans -1)
    if (Pos_Fin>=Longueur() || Pos_Fin<0)
        Pos_Fin=Longueur()-Trouver.Longueur()+1;
    if (Pos_Debut>=Longueur() || Pos_Debut<0)
    {
        Pos_Debut=Longueur()-Trouver.Longueur();
        Pos_Fin--;
    }

    //Sens
    int Increment;
    if (Pos_Fin-Pos_Debut>=0)
        Increment=1;
    else
        Increment=-1;

    //Verifie que c'est possible
    if (((Pos_Fin-Pos_Debut)*Increment-1)<0)
        return -1;

    //recherche
    int Offset;
    char* C1=new char[Longueur()+1];
    strcpy(C1, Chaine);
    char* C2=new char[Trouver.Longueur()+1];
    strcpy(C2, Trouver.Chaine);
    if (!(Options & CCHAINE_CASSE))
    {
        strlwr(C1);
        strlwr(C2);
    }
    for (int Pos=Pos_Debut; Pos!=Pos_Fin; Pos+=Increment)
        if (C1[Pos]==C2[0]) //1er char trouve
        {
            //Test si tous les char sont OK
            for(Offset=1; Offset<Trouver.Longueur(); Offset++)
                if(C1[Pos+Offset]!=C2[Offset])
                    Offset=Trouver.Longueur()+1;
            if (Offset<=Trouver.Longueur()) //si tout est bon
            {
                delete[] C1;
                delete[] C2;
                return Pos; //retourne le valeur
            }
        }
    delete[] C1;
    delete[] C2;
    return -1; //pas trouve
}

//---------------------------------------------------------------------------
//Recherche entre 2 positions
Ztring Ztring::RechercherEntre (const Ztring &Debut, const Ztring &Fin, int Pos_Debut, int Pos_Fin, int Options) const
{
    //Test integrite
    if (!Chaine || !Chaine[0])
        return "";

    //Recherche Dbut et integrite
    int I_Debut=Rechercher(Debut, Pos_Debut, Pos_Fin, Options);
    if (I_Debut<0)
        return "";

    //gestion fin NULL
    if (Fin=="")
        return Chaine+I_Debut+Debut.Longueur();

    //Copie de Debut jusqu'a la fin de la chaine
    char* T1=new char[Longueur()+1];
    strcpy (T1, Chaine+I_Debut+Debut.Longueur());

    //Recherche fin
    int I_Fin=Rechercher(Fin, I_Debut+Debut.Longueur());
    if (I_Fin>=0)
        T1[I_Fin-I_Debut-Debut.Longueur()]='\0'; //coupe a la fin

    //Creation
    Ztring C1(T1);
    delete T1;
    return C1;
}

//---------------------------------------------------------------------------
// Retourne une partie de la chaine
Ztring Ztring::SousChaine (int Pos_Debut, int Nb_Char) const
{
    //Test integrite
    if (!Chaine || !Chaine[0])
        return "";
    if (Pos_Debut<0 || Pos_Debut>=Longueur())
        return "";

    //Chaine temporaire
    char* C1=new char[Longueur() - Pos_Debut +1];
    char* Chaine2 = Chaine + Pos_Debut;
    if (Nb_Char>Longueur()-Pos_Debut)
        Nb_Char=Longueur()-Pos_Debut;
    if (Nb_Char<0)
        strcpy (C1, Chaine2);
    else
    {
           strncpy(C1, Chaine2, Nb_Char);
        C1[Nb_Char]='\0'; //strncpy ne le met pas tout seul???
    }

    //Retourne un Ztring
    Ztring C2 (C1);
    delete[] C1;
    return C2;
}

*/
//---------------------------------------------------------------------------
//Remplacement
int Ztring::FindAndReplace (const std::string &ToFind, const std::string &ReplaceBy, size_t Pos, ztring_t Options)
{
   size_t Count=0;
   size_t Milieu = Pos;
   while ((Milieu=find(ToFind, Milieu)) != std::string::npos)
   {
      replace(Milieu, ToFind.length(), ReplaceBy);
      Milieu += ReplaceBy.length();
      Count++;
   }

    return Count;
}

/*
//---------------------------------------------------------------------------
//Remplace tout et renvoie la chaine
Ztring Ztring::Reformater_Sortie (const Ztring &Trouver, const Ztring &RemplacerPar, int Pos_Debut, int Pos_Fin, int Casse)
{
    Ztring Retour=Chaine;
    while(Retour.Remplacer(Trouver, RemplacerPar, Pos_Debut, Pos_Fin, Casse)>=0);
    return Retour;
}


//---------------------------------------------------------------------------
//Comptage du nombre d'occurence
int Ztring::Compter(const Ztring &ACompter, int Casse) const
{
    //Test integrite
    if (!Chaine || !Chaine[0])
        return -1;

    int Compteur=0;
    int ACompter_Taille=ACompter.Longueur();
    //Boucle
    int I1=Longueur()-ACompter_Taille;
    for (int I2=0; I2<=I1; I2++)
        if (strnicmp(&Chaine[I2], ACompter.Chaine, ACompter_Taille)==0)
            Compteur++;
    return Compteur;
}


//---------------------------------------------------------------------------
//Comptage du nombre d'occurence
bool Ztring::Comparer (const Ztring &AComparer, const Ztring &Comparateur, int Options) const
{
    //Test integrite
    if (!Chaine || !Chaine[0])
        return -1;

    Ztring Comp (Comparateur);

    //Gestion entiers
    if (Comp.Rechercher("INT"))
    {
        //tout mettre en minuscules!
        int I1=this->ToInt();
        int I2=AComparer.ToInt();
        Comp.Remplacer("INT", "");
        if (Comp=="==") return (I2==I1);
        if (Comp=="<") return (I1<I2);
        if (Comp=="<=") return (1<=I2);
        if (Comp==">=") return (I1>=I2);
        if (Comp==">") return (I1>I2);
        if (Comp=="!=" || Comp=="<>") return (I1!=I2);
        return false;
    }

    //Gestion des options
    Ztring C1 (Chaine);
    Ztring C2 (AComparer);
    if (Comp.Rechercher("i"))
    {
        //tout mettre en minuscules!
        C1.ToMin();
        C2.ToMaj();
        Comp.Remplacer("i", "");
    }

    //Gestion caracteres
    if (Comp=="==") return (C1==C2);
    if (Comp=="IN") return (C1.Rechercher(C2));
    if (Comp=="<") return (C1<C2);
    if (Comp=="<=") return (C1<=C2);
    if (Comp==">=") return (C1>=C2);
    if (Comp==">") return (C1>C2);
    if (Comp=="!=" || Comp=="<>") return (C1!=C2);
    return false;
}
*/
//***************************************************************************
// Traitements
//***************************************************************************

//---------------------------------------------------------------------------
//Mise en minuscules
Ztring* Ztring::MakeLowerCase()
{
    std::transform(begin(), end(), begin(), tolower);
    return this;
}

//---------------------------------------------------------------------------
// Mise en majuscules
Ztring* Ztring::MakeUpperCase()
{
    std::transform(begin(), end(), begin(), toupper);
    return this;
}

//---------------------------------------------------------------------------
// Remove leading whitespaces from a string
Ztring* Ztring::TrimLeft()
{
#if !defined(_MSC_VER) //TODO : Microsft don't support that???
    std::string::const_iterator iter = std::find_if(begin(), end(), std::not1(std::ptr_fun(isspace)));
    if(iter == end())
		iter = begin();
	assign (iter, end()-iter);
#else
	size_t First=0;
	while (operator[](First)!=' ')
		First++;
	assign (c_str()+First);
#endif
	return this;
}

//---------------------------------------------------------------------------
// Remove trailing whitespaces from a string
Ztring* Ztring::TrimRight()
{
#if !defined(_MSC_VER) //TODO : Microsft don't support that???
	std::string::const_reverse_iterator iter = std::find_if(rbegin(), rend(), std::not1(std::ptr_fun(isspace)));
	assign (begin(), iter.base()-begin());
#else
	size_t Last=size();
	while (operator[](Last)!=' ')
		Last--;
	assign (c_str(), Last);
#endif
	return this;
}

//---------------------------------------------------------------------------
// Remove leading and trailing whitespaces from a string
Ztring* Ztring::Trim()
{
    TrimLeft();
    TrimRight();
    return this;
}

//---------------------------------------------------------------------------
// Quotes a string
Ztring* Ztring::Quote()
{
    assign(std::string("\"")+c_str()+"\"");
    return this;
}


/*
//---------------------------------------------------------------------------
//Padding
Ztring *Ztring::Pad(int Taille, char Char)
{
    //Test integrite
    if (Longueur()>=Taille)
        return this;

    //Copie
    char* C1=new char[Taille+1];
    if (Chaine)
        strcpy (C1, Chaine);

    //Remplissage
    for (int Pos=Longueur(); Pos<Taille; Pos++)
        C1[Pos]=Char;
    C1[Taille-1]='\0';

    //Inversion
    delete[] Chaine;
    Chaine=C1;

    return this;
}

//***************************************************************************
// Informations
//***************************************************************************

//---------------------------------------------------------------------------
//Longueur
int Ztring::Longueur() const
{
    //Test integrite
    if (!Chaine)
        return 0;

    return strlen(Chaine);
}

*/
} //namespace


