// ZenLib::ZtringList - vector<std::string> is better
// Copyright (C) 2002-2003 Jrme Martinez, Zen@MediaArea.net
//
// This software is provided 'as-is', without any express or implied
// warranty.  In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//    claim that you wrote the original software. If you use this software
//    in a product, an acknowledgment in the product documentation would be
//    appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//    misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.
//
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Version 0.0.1
//
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//---------------------------------------------------------------------------
#include <ZenLib/ZtringList.h>
//---------------------------------------------------------------------------

namespace ZenLib
{

static Ztring ZtringList_Bidon;

//***************************************************************************
// Gestion de la classe
//***************************************************************************

//---------------------------------------------------------------------------
// Constructeurs
void ZtringList::CChaine_Tableau1_Commun (int n, const Ztring &Separateurr, const Ztring &Source)
{
    //Gestion n
    reserve(n);

    //Gestion separateur
    Separateur=Separateurr;

    //Gestion Source
    if (Source!="")
        Ecrire(Source);
}

ZtringList::ZtringList (int n, const Ztring &Separateurr, const Ztring &Source)
{
    CChaine_Tableau1_Commun (n, Separateurr, Source);
}

ZtringList::ZtringList(const ZtringList &Source)
{
    CChaine_Tableau1_Commun (Source.size(), Source.Separateur, Source.Lire());
}

ZtringList::ZtringList (const Ztring &Source)
{
    CChaine_Tableau1_Commun (0, ";", Source);
}

ZtringList::ZtringList (const char* Source)
{
    CChaine_Tableau1_Commun (0, ";", Source);
}

//---------------------------------------------------------------------------
// Destructeurs
ZtringList::~ZtringList()
{
}

//***************************************************************************
// Operateurs
//***************************************************************************

//---------------------------------------------------------------------------
//Operateur =
ZtringList &ZtringList::operator = (const ZtringList &Source)
{
    clear();
    Ztring C=Separateur;
    CChaine_Tableau1_Commun (Source.capacity(), Source.Separateur, Source.Lire());
    Separateur=C;
    return *this;
}


//---------------------------------------------------------------------------
//Operateur ==
bool ZtringList::operator == (const ZtringList &Source) const
{
    return (Lire()==Source.Lire());
}

//---------------------------------------------------------------------------
//Operateur !=
bool ZtringList::operator != (const ZtringList &Source) const
{
    return (!(Lire()==Source.Lire()));
}


//---------------------------------------------------------------------------
//Operateur +=
ZtringList &ZtringList::operator += (const Ztring &Source)
{
    push_back(Source);
    return *this;
}

//---------------------------------------------------------------------------
// Operateur ()
Ztring &ZtringList::operator () (size_t Pos)
{
    if (Pos>=size())
        Ecrire("", Pos);

    return operator [] (Pos);
}

/*Ztring &ZtringList::operator () (size_t Pos) const
{
    if (Pos>=size())
        return ZtringList_Bidon ;

    std::iterator<Ztring> A=begin();

    return operator [] (Pos);
}*/
/*
//---------------------------------------------------------------------------
// Operateur ()
CChaine &ZtringList::operator [] (int Pos)
{
    //Test integrite
    if (Pos<0)
        Pos=0;
    Nettoyer();

    while (Pos>=Taille_Max)
        Taille_Max_Set(Taille_Max*2);

    if (!Chaine[Pos])
        Chaine[Pos]=new CChaine;
    if (Pos>=Taille)
        Taille=Pos+1;

    return *Chaine[Pos];
}

CChaine &ZtringList::operator [] (int Pos) const
{
    //Test integrite
    if (Pos<0)
        Pos=0;
    if (Pos>=Taille_Get())
        Pos=Taille_Get()-1;

    //Gestion Tableau vide ou chaine vide
    if (Taille_Get()==0 || !Chaine[Pos])
        return Chaine_Tableau1_Bidon ;

    return *Chaine[Pos];
}

//***************************************************************************
// Edition
//***************************************************************************

*/
//---------------------------------------------------------------------------
// Lecture
Ztring ZtringList::Lire () const
{
    Ztring Retour;
    for (size_t Pos=0; Pos<size(); Pos++)
    {
        Retour+=Lire(Pos);
        Retour+=Separateur;
    }
    if (Retour.size()>=Separateur.size())
        Retour[Retour.size()-Separateur.size()]='\0';

    return Retour;
}

Ztring ZtringList::Lire (size_t Pos) const
{
    //Test integrite
    if (Pos>=size())
        return "";

    return operator [] (Pos);
}

//---------------------------------------------------------------------------
// Ecriture
int ZtringList::Ecrire(const Ztring &AEcrire)
{
    clear();
    if (!&AEcrire || !AEcrire.size())
        return 1;

    size_t Pos=0;
    size_t PosC=0;
    bool Fini=false;
    Ztring C1;

    do
    {
        C1=AEcrire.SubString("", Separateur, PosC, Ztring_AddLastItem);
        Ecrire(C1, Pos);
        Pos++;
        PosC+=C1.size()+Separateur.size();
        if (PosC>=AEcrire.size())
            Fini=true;
    }
    while (!Fini);

    return 1;
}


int ZtringList::Ecrire(const Ztring &AEcrire, size_t Pos)
{
    if (!capacity())
        reserve(1);

    //Reservation de ressources
    while (Pos>=capacity())
        reserve(capacity()*2);

    //ecriture
    if (Pos>=size())
    {
        while (Pos>size())
            push_back ("");
        push_back(AEcrire);
    }
    else
        operator [] (Pos)=AEcrire;

    return 1;
}

/*
int ZtringList::EcrireEx(const CChaine &AEcrire, int Pos)
{
    if (AEcrire=="")
        return 1; //Pas de changements a faire
    else
        return Ecrire (AEcrire, Pos);
}

//---------------------------------------------------------------------------
// Ajout d'un ligne
int ZtringList::Ajouter (const CChaine &AAjouter)
{
    //Test integrite
    if (!AAjouter.Longueur())
        return 1;
    Nettoyer();
    
    return Ecrire(AAjouter, Taille);
}

//---------------------------------------------------------------------------
// Insertion
int ZtringList::Inserer (const CChaine &AInserer, int Pos)
{
    //Test integrite
    if (Pos<0)
        return -10;
    Nettoyer();

    if (Taille>=Taille_Max)
        Taille_Max_Set(Taille_Max*2);

    int I1=Taille;
    while (I1>Pos)
    {
        Chaine[I1]=Chaine[I1-1];
        I1--;
    }
    Chaine[Pos]=NULL;
    Taille++;

    Ecrire(AInserer, Pos);

    return 1;
}

//---------------------------------------------------------------------------
// Permutation de deux lignes
int ZtringList::Permuter (int Pos_A, int Pos_B)
{
    //Test integrite
    if (Pos_A<0 || Pos_B<0)
        return -1;
    Nettoyer();

    //Gestion de la taille du tableau
    int Pos_Max;
    if (Pos_A>Pos_B)
        Pos_Max=Pos_A;
    else
        Pos_Max=Pos_B;
    while (Pos_Max>=Taille_Max)
        Taille_Max_Set(Taille_Max*2);

    CChaine* C1=Chaine[Pos_A];
    Chaine[Pos_A]=Chaine[Pos_B];
    Chaine[Pos_B]=C1;

    if (Pos_Max>=Taille)
        Taille=Pos_Max+1;

    return 1;
}

//---------------------------------------------------------------------------
// Suppression
int ZtringList::Supprimer ()
{
    Nettoyer();
    
    for (int Pos=0; Pos<Taille; Pos++)
    {
        delete Chaine[Pos];
        Chaine[Pos]=NULL;
    }
    Taille=0;

    return 1;
}

int ZtringList::Supprimer (int Pos)
{
    Nettoyer();
    if (Taille==0)
        return 2;

    for (int I1=Pos+1; I1<Taille; I1++)
        Chaine[I1-1]=Chaine[I1];
    Taille--;
    Chaine[Taille]=NULL;

    return 1;
}


//---------------------------------------------------------------------------
// Tri
int ZtringList::Trier (int Options)
{
    Nettoyer();
    int Pos=1;
    while (Pos<Taille && *Chaine[Pos]!="")
    {
        if (*Chaine[Pos]<*Chaine[Pos-1])
        {
            //Invers
            Permuter(Pos, Pos-1);
            Pos=0;
        }
        Pos++;
    }
    return 1;
}

*/
//---------------------------------------------------------------------------
// Recherche
size_t ZtringList::Rechercher (const Ztring &ARechercher, size_t Pos, const Ztring &Comparaison, int Options) const
{
    if (Pos>=size())
        return -1;

    while (Pos<size() && (
                   Comparaison=="=="   && operator [] (Pos)!=ARechercher
                || Comparaison=="COMP" && stricmp(operator [] (Pos).Char(), ARechercher.Char())!=0
                || Comparaison=="IN"   && operator [] (Pos).find(ARechercher)==-1
                || Comparaison=="<"    && operator [] (Pos).Convert_int32()>=ARechercher.Convert_int32()
                || Comparaison=="<="   && operator [] (Pos).Convert_int32()>ARechercher.Convert_int32()
                || Comparaison==">="   && operator [] (Pos).Convert_int32()<ARechercher.Convert_int32()
                || Comparaison==">"    && operator [] (Pos).Convert_int32()<=ARechercher.Convert_int32()
                || Comparaison=="<>"   && operator [] (Pos)==ARechercher
                || Comparaison=="!="   && operator [] (Pos)==ARechercher
                ))
        Pos++;
    if (Pos>=size())
        return -1;
    return Pos;
}
/*
//***************************************************************************
// Informations
//***************************************************************************

//---------------------------------------------------------------------------
// Recuperation de la taille
int ZtringList::Taille_Get () const
{
    int Pos=Taille-1;
    while (Pos>=0 && (!Chaine[Pos] || !Chaine[Pos]->Longueur()))
        Pos--;
    return Pos+1;
}

//---------------------------------------------------------------------------
// Modification manuelle de la taille du tableau
int ZtringList::Taille_Max_Set (int NouvelleTaille)
{
    if (NouvelleTaille<0)
        return -1;

    if (NouvelleTaille==0)
        NouvelleTaille=1;

    if (NouvelleTaille==Taille_Max)
        return 1;

    //Init nouvelle chaine                                                  
    int TailleMax;
    if (NouvelleTaille<Taille_Max)
        TailleMax=NouvelleTaille;
    else
        TailleMax=Taille_Max;
    Taille_Max=NouvelleTaille;
    CChaine** C1=new CChaine*[Taille_Max];

    //Transfert
    for (int I1=0; I1<TailleMax; I1++)
        C1[I1]=Chaine[I1];
    for (int I2=TailleMax; I2<Taille_Max; I2++)
        C1[I2]=NULL;
    delete[] Chaine;

    //Suppression ancien
    Chaine=C1;

    return 1;
}

*/
//---------------------------------------------------------------------------
// Modificateur du separateur
int ZtringList::Separateur_Set (const Ztring &NouveauSeparateur)
{
    Separateur=NouveauSeparateur;
    return 1;
}

//---------------------------------------------------------------------------
// Return the length of the longest string in the list.
int ZtringList::MaxStringLength ()
{
   size_t Max = 0;
   for (ZtringList::const_iterator it=begin(); it!=end(); ++it)
      if (it->size()>Max)
         Max=(*it).size();
   return Max;
}

/*
//---------------------------------------------------------------------------
// Nettoyage
int ZtringList::Nettoyer ()
{
    if (!Chaine || Taille==0)                                
        return 1;

    int Pos=Taille-1;
    while (Pos>=0 && (!Chaine[Pos] || !Chaine[Pos]->Longueur()))
    {
        delete Chaine[Pos];
        Chaine[Pos]=NULL;
        Taille--;
        Pos--;
    }
    return 1;
}
*/
} //namespace






