// ZenLib::ZtringListListF - ZtringListList with file load/save
// Copyright (C) 2002-2003 Jrme Martinez, Zen@MediaArea.net
//
// This software is provided 'as-is', without any express or implied
// warranty.  In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//    claim that you wrote the original software. If you use this software
//    in a product, an acknowledgment in the product documentation would be
//    appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//    misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.
//
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Version 1.0.0
//
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// ToDo
// Rajouter les tests d'integrite et sur les fichiers disque
//
// 1.0.0.001 (Zen@MediaArea.net, 2002-10-27)
// Nettoyages
//
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//---------------------------------------------------------------------------
#include <ZenLib/ZtringListListF.h>
#include <ZenLib/FileT.h>
#include <Windows.h>
//---------------------------------------------------------------------------

namespace ZenLib
{

//***************************************************************************
// Gestion de la classe
//***************************************************************************

//---------------------------------------------------------------------------
// Constructeur
ZtringListListF::ZtringListListF(int Taille2, int TailleBase1, const Ztring &Separateur1, const Ztring &Separateur2, const char *Source)
:ZtringListList (Taille2, TailleBase1, Separateur1, Separateur2, Source)
{
	Backup_Nb_Max=0;
	Backup_Nb=0;
    Sauvegarde=true;
}

//---------------------------------------------------------------------------
// Destructeur
ZtringListListF::~ZtringListListF()
{
}

//***************************************************************************
// Gestion disque
//***************************************************************************

//---------------------------------------------------------------------------
// Chargement global
int ZtringListListF::Charger (const Ztring &Fichier)
{
    if (Fichier!="")
        Nom=Fichier;
    int I1=-1;

    Ecrire("");

    if (Nom.find(".csv")!=-1)
        I1=CSV_Charger();
    if (Nom.find(".cfg")!=-1)
        I1=CFG_Charger();
    if (I1>=0)
    {
        Backup_Nb=0; //mettre ici le code pour trouver le nb de backup
        return Backup_Nb;
    }
    else
        return I1;
}

//---------------------------------------------------------------------------
// Chargement CSV
int ZtringListListF::CSV_Charger ()
{
    FileT *F= new FileT();
    if (F->Ouvrir (Nom)<0)
        return -1;

    Ztring SeparateurT=Separateur[1];
    Separateur[1]=";";
    while (!F->Eof())
        push_back(F->Lire());
    delete F;
    Separateur[1]=SeparateurT;
    return 1;
}

//---------------------------------------------------------------------------
// Chargement CFG
int ZtringListListF::CFG_Charger ()
{
    FileT *F= new FileT();
    if (F->Ouvrir (Nom)<0)
    {
        delete F;
        return -1;
    }

    Ztring SeparateurT=Separateur[1];
    Separateur[1]=";";

    Ztring Lu;
    Ztring Propriete, Valeur, Commentaire;

    while (!F->Eof())
    {
        if ((Lu=F->Lire()).size()==0)
        {
            delete F;
            return -2;
        }
        if (Lu.find("=")>0)
        {
            //Obtention du nom
            Propriete=Lu.SubString("", "=");
            NettoyerEspaces(Propriete);
            //Obtention de la valeur
            Valeur=Lu.SubString("=", ";", 0, Ztring_AddLastItem);
            NettoyerEspaces(Valeur);
        }
        //Obtention du commentaire
        Commentaire=Lu.SubString(";", "", 0, Ztring_AddLastItem);
        NettoyerEspaces(Commentaire);
        //Ecriture
        push_back((Propriete+";"+Valeur+";"+Commentaire).c_str()); //Visual C++ 6 is old...
    }
    delete F;
    Separateur[1]=SeparateurT;
    return 1;
}

//---------------------------------------------------------------------------
// Sauvegarde globale
int ZtringListListF::Sauvegarder (const Ztring &Fichier)
{
    //Gestion de l'annulation de la sauvegarde
    if (!Sauvegarde)
        return Backup_Nb;

    if (Fichier!="")
        Nom=Fichier;
    /*else
    {
        Ztring C1=Nom;
        Nom=C1;
    } */      
    //Gestion des backups
    Backup_Nb=0;
    int I2;
    for (int I1=Backup_Nb_Max-1; I1>=0; I1--)
    {
        Ztring Z1=Nom+".sav"; Z1+=I1; //Visual C++ 6 patch
        Ztring Z2=Nom+".sav"; Z2+=(I1+1); //Visual C++ 6 patch
        I2=MoveFileEx(Z1.c_str(), Z2.Char(), MOVEFILE_REPLACE_EXISTING);
        /*I2=GetLastError();
        char CC[1000];
        I2=FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, I2, 0, CC, 1000, NULL);*/
        if (I2 && !Backup_Nb)
            Backup_Nb=I2;
    }
    if (Backup_Nb_Max)
    {
        Ztring Z1=Nom+".sav0"; //Visual C++ 6 patch 
		MoveFileEx(Nom.c_str(), Z1.c_str(), MOVEFILE_REPLACE_EXISTING);
        Backup_Nb++;
    }

    I2=-1;
    if (Nom.find(".csv")!=-1)
        I2=CSV_Sauvegarder();
    if (Nom.find(".cfg")!=-1)
        I2=CFG_Sauvegarder();

    if (I2>=0)
    {
        return Backup_Nb;
    }
    else
        return I2;
}

//---------------------------------------------------------------------------
// Sauvegarde CSV
int ZtringListListF::CSV_Sauvegarder ()
{
    //Sauvegarde
    FileT *F= new FileT();
    if (F->Ouvrir (Nom, true)<0)
    {
        delete F;
        return -1;
    }
    Ztring SeparateurT=Separateur[1];
    Separateur[1]=";";
    uint32 I1=0;
    while (I1<size())
    {
        F->Ecrire(Lire(I1));
        I1++;
    }
    delete F;
    Separateur[1]=SeparateurT;
    return 1;
}

//---------------------------------------------------------------------------
// Sauvegarde CFG
int ZtringListListF::CFG_Sauvegarder ()
{
    FileT *F= new FileT();
    if (F->Ouvrir (Nom, true)<0)
    {
        delete F;
        return -1;
    }

    Ztring AEcrire;
    Ztring Propriete, Valeur, Commentaire;

    Ztring SeparateurT=Separateur[1];
    Separateur[1]=";";
    uint32 I1=0;
    while (I1<size())
    {
        Propriete=Lire(I1, 0);
        Valeur=Lire(I1, 1);
        Commentaire=Lire(I1, 2);
        if (Propriete!="")
        {
            AEcrire=Propriete+" = ";
            if (Valeur!="")
                AEcrire+=Valeur+" ";
        }
        if (Commentaire!="")
            AEcrire+="; "+Commentaire;
        F->Ecrire(AEcrire);
        I1++;
    }
    delete F;
    Separateur[1]=SeparateurT;
    return 1;
}

//---------------------------------------------------------------------------
// Annulation
int ZtringListListF::Annuler ()
{
    Ztring Z1=Nom+".sav0"; //Visual C++ 6 patch 
	MoveFileEx(Z1.c_str(), Nom.c_str(), MOVEFILE_REPLACE_EXISTING);
    for (int I1=1; I1<=Backup_Nb; I1++)
	{
		Ztring Z2=Nom+".sav"; Z2+=I1; //Visual C++ 6 patch 
		Ztring Z3=Nom+".sav"; Z3+=(I1-1); //Visual C++ 6 patch 
        MoveFileEx(Z2.Char(), Z3.Char(), MOVEFILE_REPLACE_EXISTING);
    }
	Ecrire("");
    return CSV_Charger();
}

//***************************************************************************
// Divers
//***************************************************************************

//---------------------------------------------------------------------------
// Nettoyage
int ZtringListListF::NettoyerEspaces (Ztring &ANettoyer)
{
    uint32 Debut=0;
    while (Debut<ANettoyer.size() && ANettoyer[Debut]==' ')
        Debut++;
    uint32 Fin=ANettoyer.size()-1;
    while (Fin!=-2 && ANettoyer[Fin]==' ')
        Fin--;
    if (Fin>=Debut)
        ANettoyer=ANettoyer.substr(Debut, Fin-Debut+1);
    else
        ANettoyer="";
    return 1;
}

//---------------------------------------------------------------------------
//
int ZtringListListF::Sauvegarde_Set (bool Sauvegardee)
{
    Sauvegarde=Sauvegardee;
    Sauvegarder();
    return 1;
}

//---------------------------------------------------------------------------
//
int ZtringListListF::Backup_Set (int Nombre)
{
    if (Nombre>=0)
        Backup_Nb_Max=Nombre;
    return Nombre;
}

} //Namespace
